//+------------------------------------------- 
//| BatteryInfo Widget
//+------------------------------------------- 
//| $LastChangedDate$
//+-------------------------------------------
//| Info:
//| Copyright (c) 2005 Ben Barnett
//| widgets@menial.co.uk
//+-------------------------------------------


/***
* Sets a few vars
***/
var widgetName = 'BatteryInfo';
var widgetVersion = '';
var debug = false;

// Options
var updateInterval = 10000;
var showPercentage = true;
var showTime = false;
var lastUpdate = 0;

// Objects
var updateTimer = null;
var batteryInfo = new Array();


/***
* Sets up widget after being run
*
* Sets onshow and onhide events
***/
function setup() {
	if (debug) alert('DEBUG: Initializing widget');
	if (window.widget)
	{
	widget.onshow = onshow;
	widget.onhide = onhide;
	}
}


/***
* Sets up widget for showing
*
* Loads localized strings
* Gets preferences
* Updates the back to show current prefs
* Shows the battery status
* Sets the loop to update the status
***/
function onshow() {
	if (debug) alert('DEBUG: Showing widget');
	widgetVersion = getKeyValue('Info.plist', 'CFBundleVersion');
	loadLocalizedStrings();
	getPrefs();
	updatePrefs();
	showStatus();
	if (updateTimer == null)
		{
		updateTimer = setInterval("showStatus()", Number(updateInterval));
		}
	else
		{
		clearInterval(updateTimer);
		updateTimer = null;
		
		updateTimer = setInterval("showStatus()", Number(updateInterval));
		}
}


/***
* Sets widget for hiding
*
* Destroys the loop
* Saves preferences
***/
function onhide() {
	if (debug) alert('DEBUG: Hiding widget');
	if (updateTimer != null)
	{
	clearInterval(updateTimer);
	updateTimer = null;
	}
	checkLastUpdate();
}


/***
* Displays the battery status
*
* Gets info
* Gets percentage from info
* Selects image number from percentage
* Prepends image number with relevant status choice
*
* Displays the image
***/
function showStatus() {
if (debug) alert('DEBUG: Starting display of battery info');
if (window.widget)
	{
	systemType = getBatteryStatus();

	if (systemType == 'desktop')
		{
		if (debug) alert('DEBUG: Desktop system. Widget won\'t work!');
		document.getElementById("status").src = "images/desktop_notice.png";
		document.getElementById("info").style.display ='none';
		document.getElementById("badge").style.display ='none';
		resizeTo(128, 128);
		if (debug) alert('DEBUG: Finished display of battery info');
		return;
		}
	
	var image = selectImage(batteryInfo['percentage']);
	
	// If we can't choose an image
	if (image == 'no_image')
		{
		document.getElementById("status").src = "images/0.png";
		}
	// Otherwise, display it!
	else
		{
		
		switch (batteryInfo['status'])
			{
			case 'discharging':
			document.getElementById("status").src = "images/"+image;
			break;	
			
			case 'charging':
			document.getElementById("status").src = "images/charging_"+image;
			break;
			
			case 'charged':
			document.getElementById("status").src = "images/charged.png";
			break;
			
			case 'AC attached':
			
			if(batteryInfo['remaining'] == 'not_charging')
				{
					// Battery is CHARGED
				document.getElementById("status").src = "images/charged.png";
				}	
			else
				{
					// Battery is CHARGING
				document.getElementById("status").src = "images/charging_"+image;
				}
			break;
	
			case 'removed':
			document.getElementById("status").src = "images/0.png";
			break;
			
			default:
			// If all else fails. Display an empty battery
			document.getElementById("status").src = "images/0.png";
			break;
			}
		}
		
	if (showPercentage == true)
		{
		// MWW - One hack to rule them all and in the widget bind them - all other 100% percentage adjusters removed elsewhere!
		percentValue = Number(batteryInfo['percentage']);
		if (percentValue > 99)
			{
			document.getElementById('badgeleft').style.background = 'url(images/badge/9.png)';
			document.getElementById('badgeright').style.background = 'url(images/badge/9.png)';
			//document.getElementById('badgeleft').style.background = 'url(images/badge/f.png)';
   			//document.getElementById('badgeright').style.background = 'url(images/badge/l.png)';	
			}
		else if (percentValue > 9)
			{
			document.getElementById('badgeleft').style.background = 'url(images/badge/'+batteryInfo['percentage'].charAt(0)+'.png)';
			document.getElementById('badgeright').style.background = 'url(images/badge/'+batteryInfo['percentage'].charAt(1)+'.png)';	
			}
		else if (percentValue  < 10) 
			{
			document.getElementById('badgeleft').style.background = 'url(images/badge/blank.png)';
			document.getElementById('badgeright').style.background = 'url(images/badge/'+batteryInfo['percentage']+'.png)';
			}
		else
			{
			document.getElementById('badge').style.display = 'none';
			}
		} // END IF( SHOW PERCENTAGE )

	if (showTime == true)
		{
		// Battery Full
		if (batteryInfo['remaining'] == 'not_charging')
			{
			document.getElementById('infotext').innerText = getLocalizedString('batteryfull');
			}
		
		// Calculating Time
		else if (batteryInfo['remaining'] == 'no_estimate')
			{
			document.getElementById('infotext').innerText = getLocalizedString('noestimate');
			}
		// We have a time..
		else
			{
			// ...remaining
			if (batteryInfo['status'] == 'discharging')
				{
				document.getElementById('infotext').innerText = batteryInfo['remaining']+' '+getLocalizedString('remaining');
				}
			// ...or until fully charged
			else if (batteryInfo['status'] == 'charging')
				{
				document.getElementById('infotext').innerText = batteryInfo['remaining']+' '+getLocalizedString('untilfull');
				}
			// There is no battery installed	
			else if (batteryInfo['status'] == 'removed')
				{
				document.getElementById('infotext').innerText = getLocalizedString('nobattery');
				
				}	
			
			}
		}	// End if(showTime)
	
	}
if (debug) alert('DEBUG: Finished display of battery info');
} // End function


/***
* Gets Battery Info
*
* Gets raw battery info from pmset
***/
function getBatteryStatus() {
	if (debug) alert('DEBUG: Getting raw battery info');
	
	// Get raw battery info	
	var resultString = widget.system("/usr/bin/pmset -g batt", null).outputString;
	
	// Split at newline
	tempString = resultString.split(/\n/);

	//tempString[1] = '-InternalBattery-0	(removed)';
	//tempString[2] = '-InternalBattery-1	100%; AC attached; not charging';
	// Check if we're a desktop system
	if ( tempString[1] == '' || tempString[1] == null )
		{
		return 'desktop';
		}
		
	// And see if a second battery exists
	if ( (tempString[2] != null) && (tempString[2] != '') )
		{
		batteryInfoOne = processBatteryInfo(tempString[1]);
		batteryInfoTwo = processBatteryInfo(tempString[2]);
		
		if ( (batteryInfoOne['status'] != 'removed') && (batteryInfoTwo['status'] != 'removed') ) 
			{
			// Both batteries Installed

			batteryInfo['percentage'] = Math.round(( Number(batteryInfoOne['percentage']) + Number(batteryInfoTwo['percentage']) ) / 2);
			batteryInfo['percentage'] += '';
			
			// WAS: batteryInfo['status'] = batteryInfoOne['status'];
			if (batteryInfoOne['status'] == batteryInfoTwo['status'])
				{
				batteryInfo['status'] = batteryInfoOne['status'];
				}
			else if ( (batteryInfoOne['status'] == 'charging') || (batteryInfoTwo['status'] == 'charging') ) 
				{
				batteryInfo['status'] = 'charging';
				}
			else if ( (batteryInfoOne['status'] == 'discharging') || (batteryInfoTwo['status'] == 'discharging') ) 
				{
				batteryInfo['status'] = 'discharging';
				}
			else
				{
				// Not sure what to fall back so, so this will have to do for now!
				batteryInfo['status'] = batteryInfoOne['status'];
				}
			
			
			if ( (batteryInfoOne['remaining'] != 'not_charging') && (batteryInfoOne['remaining'] != 'no_estimate') && (batteryInfoTwo['remaining'] != 'not_charging') && (batteryInfoTwo['remaining'] != 'no_estimate'))
				{
				timeOne = batteryInfoOne['remaining'].split(':');
				timeTwo = batteryInfoTwo['remaining'].split(':');
				hours = Number(timeOne['0']) + Number(timeTwo['0']);
				minutes = Number(timeOne['1']) + Number(timeTwo['1']);
				// MWW - this can produce a minutes range greater than on hour so adjust fields
				if (minutes > 59)
					{
						minutes -= 60;
						hours += 1;
					}
				batteryInfo['remaining'] = hours+ ((minutes < 10) ? ':0' : ':') +minutes;
				}
			else if ((batteryInfoOne['remaining'] == 'no_estimate') || (batteryInfoTwo['remaining'] == 'no_estimate'))
				{
				batteryInfo['remaining'] = 'no_estimate';
				// No estimate goes first as we want it to take precedence over battery full
				}
			else if ((batteryInfoOne['remaining'] == 'not_charging') || (batteryInfoTwo['remaining'] == 'not_charging'))
				{
				batteryInfo['remaining'] = 'not_charging';
				}
			else
				{
				batteryInfo['remaining'] = 'no_estimate';
				}
			} // End BOTH BATTERIES
			
		// Battery ONE installed
		else if ( (batteryInfoOne['status'] != 'removed') && (batteryInfoTwo['status'] == 'removed') )
			{
			batteryInfo = batteryInfoOne;
			}
			
		// Battery TWO installed
		else if ( (batteryInfoOne['status'] == 'removed') && (batteryInfoTwo['status'] != 'removed') )
			{
			batteryInfo = batteryInfoTwo;
			}
		
		// NEITHER batteries installed
		else if ( (batteryInfoOne['status'] == 'removed') && (batteryInfoTwo['status'] = 'removed') )
			{
			// assume empty
			batteryInfo['percentage'] = 0;
			batteryInfo['status'] = 'removed';
			batteryInfo['remaining'] = 'no_battery';
			}
		
		} // END TWO BATTERIES
	else
		{
		batteryInfo = processBatteryInfo(tempString[1]);
		}
	
return true;
}





function processBatteryInfo(line) {
	if (debug) alert('DEBUG: Processing raw battery info');
		
	temp = line.split(/\t/);
	tempInfo = new Array();
	if (temp[1] == '(removed)')
		{
		tempInfo['percentage'] = 0;
		tempInfo['status'] = 'removed';
		tempInfo['remaining'] = 'no_battery';
		return tempInfo;
		}
	
	temp = temp[1].split(';');
	tempInfo['percentage'] = temp[0].replace(';', '');
	tempInfo['percentage'] = temp[0].replace('%', '');
	
// MWW - changed badge selector to eliminate percentage adjustment need.
// Primary reason to improve dual battery accuracy but also fixes latent
// 100% charged badge display anomaly if one only battery installed.
//	if (tempInfo['percentage'] == '100')
//		{
//		tempInfo['percentage'] = '99';
//		}

	tempInfo['status'] = temp[1].replace(' ', '');
	
	// MWW - removed percentage adjustments
	
	switch (temp[2])
		{
		case ' not charging':
			tempInfo['remaining'] = 'not_charging';
		break;
		// MWW - raw dump of output shows space at end of (no estimate) string
		case ' (no estimate) ':
			tempInfo['remaining'] = 'no_estimate';
		break;
		
		default:
			temp = temp[2].split(' ');
			tempInfo['remaining'] = temp[1];
		break;
		}
	
	return tempInfo;
}





/***
* Chooses which image to load
*
* 10 images
***/
function selectImage(percentage) {
	if (debug) alert('DEBUG: Selecting battery image');
	if (percentage > 95)
		{
		return "10.png";
		}
	else if ( (percentage > 85) && (percentage <= 95) )
		{
		return "9.png";
		}
	else if ( (percentage > 75) && (percentage <= 85) )
		{
		return "8.png";
		}
	else if ( (percentage > 65) && (percentage <= 75) )
		{
		return "7.png";
		}
	else if ( (percentage > 55) && (percentage <= 65) )
		{
		return "6.png";
		}
	else if ( (percentage > 45) && (percentage <= 55) )
		{
		return "5.png";
		}
	else if ( (percentage > 35) && (percentage <= 45) )
		{
		return "4.png";
		}
	else if ( (percentage > 25) && (percentage <= 35) )
		{
		return "3.png";
		}
	else if ( (percentage > 15) && (percentage <= 25) )
		{
		return "2.png";
		}
	else if ( (percentage > 5) && (percentage <= 15) )
		{
		return "1.png";
		}
	else if (percentage <= 5)
		{
		return "0.png";
		}
	else
		{
		return "no_image"
		}
}




/***
* Flips the widget over (To BACK)
*
* Flips & Resizes
***/
function showPrefs()
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");
        
    if (window.widget)
    	{
    	document.getElementById("flip").style.opacity = 0;
    	document.getElementById("fliprollie").style.display = 'none';
    	if (showTime == true)
	    	{
	    	resizeTo(172, 135);
	    	}
    	else
    		{
    		window.resizeTo (172, 128);
    		}
        widget.prepareForTransition("ToBack");
        
    	}
    	        
    front.style.display="none";
    back.style.display="block";
        
    if (window.widget)
    	{
        setTimeout ('widget.performTransition();', 0);  
        window.resizeTo (172, 128);
    	}
}


/***
* Flips the widget over (To FRONT)
*
* Flips & Resizes
***/
function hidePrefs()
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");
    
        
    if (window.widget)
	{	
	if (showTime == true)
    	{
    	resizeTo(172, 135);
    	}
    widget.prepareForTransition("ToFront");
	}
                
    back.style.display = "none";
    front.style.display = "block";
    document.getElementById('fliprollie').style.display = 'none';
        
    if (window.widget)
	{
    setTimeout ('widget.performTransition();', 0);
    if (showTime == true)
    	{
    	resizeTo(128, 135);
    	}
    else
    	{
		resizeTo(128, 128);
    	}
	}
}



/***
* Functions to convert decimal to hex (d2h();) and 
* back again (h2d();)
***/
var hD="0123456789ABCDEF";
function d2h(d) {
var h = hD.substr(d&15,1);
while(d>15) {d>>=4;h=hD.substr(d&15,1)+h;}
return h;
}
function h2d(h) {return parseInt(h,16);}






/***
* Opens link
***/
function openUrl(url) 
	{
	if (window.widget) 
		{
		widget.openURL(url);
		}
	else 
		{
		window.open(url);
		}
}



/***
* Retrieves a key value from a specified plist
***/
function getKeyValue(plist, key) 
	{ 
	if (debug) alert('DEBUG: XMLREQ: Loading key "'+key+'" from plist "'+plist+'"');
	var xml_http = new XMLHttpRequest(); 
	xml_http.open("GET", plist, false); 
	xml_http.send(null); 

	var xml = xml_http.responseXML; 
	var keys = xml.getElementsByTagName("key"); 
	var key_value; 
    
	for (var i=0; i < keys.length; i++) 
		{ 
		if (keys[i].firstChild.data == key) 
			{ 
			value_node = keys[i].nextSibling.nextSibling; 
			key_value = value_node.firstChild.data; 
			//if (debug) alert('DEBUG: XMLREQ: Key Data: '+key_value);
			key_type = value_node.nodeName; 
			//if (debug) alert('DEBUG: XMLREQ: Node type: '+key_type);
			break; 
			} 
		} 
	return key_value; 
	}
