// http://developer.apple.com/technotes/tn2005/tn2139.html
// Javascript coded by Robert Hagenström, copyright 2006

var timerInterval = null;

var oldlist = {};
var oldsize = 0;

if (window.widget) {
    widget.onhide = onhide;
    widget.onshow = onshow;
}

function onshow() {
    if (timerInterval == null) {
        timerInterval = setInterval("update();", 1000); // tried 25 frames per second but finder only updates results after copy completion
    }
}

function onhide() {
    if (timerInterval != null) {
        clearInterval(timerInterval);
        timerInterval = null;
    }
}

function setup () {
 var position = document.getElementById("diskspace");
 var disklist = getDiskList();
 for (var n in disklist) {
	position.appendChild(createDiskDiv(disklist[n].label, disklist[n].free, disklist[n].used));
 }
 window.resizeBy(0, position.offsetHeight - oldsize);
 oldsize = position.offsetHeight;
 oldlist = disklist;
}

function update() {
 var position = document.getElementById("diskspace");
 var disklist = getDiskList();
 var newdisk;
 for (var n in disklist) { // update according to the new info
	if (oldlist[n] != undefined) { // check if the disk existed last time
//		alert("Updating: " + n);
		updateSpace(disklist[n].label, disklist[n].free, disklist[n].used); // only update the data
		delete oldlist[n]; // old list is reused as "processed entries" basically
	}
	else {
//		alert("New entry: " + n);
		newdisk = createDiskDiv(disklist[n].label, disklist[n].free, disklist[n].used); // create new disk if new
		position.appendChild(newdisk); // and right now we simply append it at the end
	}
 }

 for (var n in oldlist) { // remove all disks not found in new info
//	alert("Unhandled entires: " + n);
 		var node = document.getElementById(n);
		node.parentNode.removeChild(node);
 }
 oldlist = disklist; // new info becomes old info

 if (position.offsetHeight != oldsize) {
	window.resizeBy(0, position.offsetHeight - oldsize);
	oldsize = position.offsetHeight; // new size becomes old size
 }
}

function disk(plabel, pfree, pused) { // this is actually an object...
	this.label = plabel || "";
	this.free = pfree || "0 MB";
	this.used = pused || "0%";
}

function updateDisks() {
}

function updateSpace(label, free, used) {
	var diskfree = document.getElementById(label + "free");
	var diskused = document.getElementById(label + "used");
	diskfree.innerHTML = free;
	diskused.innerHTML = used;
	diskused.style.width = used;
}

function getDiskList () {
// var _input = [ "OS X\t55050\t16081", "XP\t20931\t20931", ""];
 var _input = widget.system("/usr/bin/osascript DiskSpace.scpt", null).outputString;
 _input = _input.split("\r");
 var count = _input.length - 1;
 var disklist = {};
 var _list, _label, _free, _used;
 for (var i=0;i<count;i++) {
	_list = _input[i].split("\t");
	_label = _list[0];
	_free = (_list[2] > 999 ? (Math.round(_list[2] / 1024 * 10) / 10) + ' GB' : _list[2] + ' MB');
	_used = Math.round(100 - (_list[2] / _list[1]) * 100) + '%';
	disklist[_label] = new disk(_label, _free, _used);
 }
 return disklist;
}

function createDiskDiv (label, free, used) { // just build according to the template
 var newdisk = document.createElement('div');
 newdisk.className = "container";
 newdisk.id = label;
 var disklabel = document.createElement('h1');
 disklabel.innerHTML = label;
 newdisk.appendChild(disklabel);
 var disklist = document.createElement('dl');
 newdisk.appendChild(disklist);
 var diskspaceinfo = document.createElement('dt');
 diskspaceinfo.className = "free";
 diskspaceinfo.innerHTML = "Free Space";
 disklist.appendChild(diskspaceinfo);
 var diskspace = document.createElement('dd');
 diskspace.className = "free";
 diskspace.id = label + "free";
 diskspace.innerHTML = free;
 disklist.appendChild(diskspace);
 var emptyspaceinfo = document.createElement('dt');
 emptyspaceinfo.className = "usage";
 emptyspaceinfo.innerHTML = "Used Space";
 disklist.appendChild(emptyspaceinfo);
 var emptyspace = document.createElement('dd');
 emptyspace.className = "usage";
 disklist.appendChild(emptyspace);
 var fullspace = document.createElement('span');
 fullspace.id = label + "used";
 fullspace.style.width = used;
 fullspace.innerHTML = used;
 emptyspace.appendChild(fullspace);

 return newdisk;
}