var maxColumns = 50;
var columns = new Array();
var url = "http://www.widgetop.com";
var maxHeight = 40;
var timeout = null;
var stats = {maxValue: 1, minValue: -1, average: 0, average2: 0, checks: 0, errors: 0, lastCheck: null, lastValue: 0};
var frequency = 15;
var errors = "";

function setup() {
  if (window.widget) {
    widgetSetup();
    fillChart();
    readPrefs();
    widget.onremove = clearPrefs;
    widget.onshow = onshow;
    widget.onhide = onhide;
    if (!widget.context) {
      document.getElementById("Widgetop").style.display = "block";
    }
  }
}

function readPrefs() {
  if (window.widget) {
    var value = widget.preferenceForKey(widget.identifier + "-url");
    if (value) {
      url = value;
    }
    //document.getElementById("Address").innerHTML = url;  #
    updateAddress();
    document.getElementById("UrlInput").value = url;  
    
    var value = widget.preferenceForKey(widget.identifier + "-frequency");
    if (value) {
      frequency = parseInt(value);
    }
    document.getElementById("Frequency").value = frequency;  
  }
}

function clearPrefs() {
  widget.setPreferenceForKey(null, widget.identifier + "-url");
  widget.setPreferenceForKey(null, widget.identifier + "-frequency");
}

function fillChart() {
  var chart = document.getElementById("Chart");
  for(var i=0; i<maxColumns; i++) {
  	column = document.createElement("div");
  	column.id = "Column" + i;
  	column.className = "column";
  	//column.innerHTML = "";
  	column.style.left = i * 4 + "px";
  	chart.appendChild(column);
  }
}

function onshow() {
  if (timeout) {
    clearTimeout(timeout);    
  }
  timeout = setTimeout(checkUrl, 1);  
}

function onhide() {
  //Don't stop on hide anymore
  /* 
  if (timeout) {
    clearTimeout(timeout);
  }  
  timeout = null;
  */
}

function checkUrl() {  
  var request = new XMLHttpRequest();
  var start = new Date().getTime();
  request.open("HEAD", url, true);
  request.onload = function() {
    urlChecked(request, start);
  }
  stats.checks++;  
  request.send(null);
  if (timeout) {
    clearTimeout(timeout);
    timeout = null;    
  }
  if (frequency != 0) {
    timeout = setTimeout(checkUrl, frequency * 60000);
  }  
}

function urlChecked(request, start) {
  var when = new Date();
  stats.lastCheck = when;
  var end = when.getTime();
  var value = end - start;    
  if (request.getResponseHeader("timing")) {
    value = parseInt(request.getResponseHeader("timing"));
  }
  value = Math.max(1, value);
  if (request.status == 200) {
    stats.lastValue = value + "ms";
    if (value > stats.maxValue) {
      stats.maxValue = value;
    }
    if (value < stats.minValue || stats.minValue <= 0) {
      stats.minValue = value;
    }
    stats.average += value;
    stats.average2 = stats.average / (stats.checks - stats.errors);
    var column = new Object();
    column.value = value;
    column.when = when;
    addColumn(column);
  }
  else {
    stats.errors++;
    document.getElementById("Background").style.backgroundImage = "url(Front_alert.png)";
    var statusText = request.statusText;
    if (statusText != null) {
      statusText = decodeURIComponent(request.statusText).replace(/\+/g, " ");
    }
    text = "<b>" + request.status + " - " + statusText + "</b><br />" + when.toLocaleTimeString() + "<p />";
    stats.lastValue = request.status + " - " + statusText;
    errors = text + errors;
    document.getElementById("InnerLog").innerHTML = errors;
    document.getElementById("InnerLog").className = null;
    var column = new Object();
    column.isError = true;    
    column.when = when;
    column.error = stats.lastValue;
    addColumn(column);
  }
}

function addColumn(column) {
  if (columns.length == maxColumns) {
    columns.splice(0, 1);
  }
  columns.push(column);
  updateStats();
  updateChart();   
}

function showContent(id) {
  document.getElementById("Chart").style.display = "none";
  document.getElementById("Stats").style.display = "none";
  document.getElementById("Log").style.display = "none";
  document.getElementById("ChartTab").className = "";
  document.getElementById("StatsTab").className = "";
  document.getElementById("LogTab").className = "";

  document.getElementById(id).style.display = "block";
  document.getElementById(id + "Tab").className = "highlight";
}

function updateStats() {
  document.getElementById("StatMax").innerHTML = stats.maxValue + " ms";
  document.getElementById("StatMin").innerHTML = stats.minValue + " ms";
  document.getElementById("StatAverage").innerHTML = stats.average2.toFixed(1) + " ms";
  document.getElementById("StatChecks").innerHTML = stats.checks;
  document.getElementById("StatErrors").innerHTML = stats.errors;
  document.getElementById("StatLastCheck").innerHTML = stats.lastCheck.toLocaleTimeString();
  document.getElementById("StatLastValue").innerHTML = stats.lastValue;
}

function updateChart() {
  for (var i = 0; i < columns.length; i++) {
    var id = (i + maxColumns - columns.length);
  	var bar = document.getElementById("Column" + id);
  	if (columns[i].isError) {
  	  bar.style.top = "0px";
  	  bar.style.height = maxHeight + "px";
  	  bar.className = "column errorColumn";
  	  bar.title = columns[i].error;
  	}
  	else {  
  	  var height = maxHeight * columns[i].value / stats.maxValue;
  	  bar.style.top = (maxHeight - height) + "px";
  	  bar.style.height = height + "px";
  	  if (columns[i].value > stats.average2 * 2.0) {
  	    bar.className = "column warnColumn";
  	  }
  	  else {
  	    bar.className = "column";
  	  }  
  	  bar.title = columns[i].value + "ms - " + columns[i].when.toLocaleTimeString();
  	}  
  }
  for (var i = 0; i < maxColumns - columns.length; i++) {
  	var bar = document.getElementById("Column" + i);  	
  	bar.style.top = "0px";//maxHeight + "px";

  	bar.style.height = "0px";
  }
  if (stats.maxValue != 0) {
    document.getElementById("Average").style.top = maxHeight - parseInt(maxHeight * stats.average2 / stats.maxValue);
  }  
}

function openUrl() {
  widget.openURL(url);
}

function reset() {
  stats.average = 0;
  stats.average2 = 0;
  stats.minValue = 0;
  stats.maxValue = 0;
  stats.checks = 0;
  stats.errors = 0;
  stats.lastCheck = null;
  stats.lastValue = 0;
  columns = new Array();
  document.getElementById("Background").style.backgroundImage = "";//null;
  errors = "";
  document.getElementById("InnerLog").innerHTML = "No Errors";
  document.getElementById("InnerLog").className = "emptyLog";
}

function updateAddress() {
  document.getElementById("Icon").style.display = "none";
  document.getElementById("Refresh").style.backgroundImage = "";//null;
  var icon = new Image();
  icon.onload = function() {
    document.getElementById("Icon").style.display = "block";
    document.getElementById("Icon").src = icon.src;    
    document.getElementById("Refresh").style.backgroundImage = "url(no)";
  }
  icon.src = url + "/favicon.ico";
  document.getElementById("Address").innerHTML = url;
}

function widgetSetup() {
  if (window.widget) {
  	var gDoneButton = new AppleGlassButton(document.getElementById("DoneButton"), 
  	                                      "Done", hideBack); 
  	var gResetButton = new AppleGlassButton(document.getElementById("ResetButton"), 
  	                                      "Reset", reset); 
	  var gInfoButton = new AppleInfoButton(document.getElementById("InfoButton"), 
	                                        document.getElementById("front"), 
	                                        "white", "white", showBack);    
  }
}

function showBack() {
    var front = document.getElementById("front");
    var back = document.getElementById("back");
 
    if(window.widget)
        widget.prepareForTransition("ToBack");
 
    front.style.display="none";
    window.resizeTo(220, 290);
    back.style.display="block";
 
    if (window.widget)
        setTimeout ('widget.performTransition();', 0);  
}

function hideBack() {
    var front = document.getElementById("front");
    var back = document.getElementById("back");
 
    if (window.widget)
        widget.prepareForTransition("ToFront");
 
    back.style.display="none";
    window.resizeTo(260, 110);
    url = document.getElementById("UrlInput").value;
    updateAddress();
    //document.getElementById("Address").innerHTML = url;
    frequency = parseInt(document.getElementById("Frequency").value);
    widget.setPreferenceForKey(url, widget.identifier + "-url");
    widget.setPreferenceForKey(frequency, widget.identifier + "-frequency");
    document.getElementById("Background").style.backgroundImage = "";//null;
    checkUrl();
    front.style.display="block";
 
    if (window.widget)
        setTimeout ('widget.performTransition();', 0);
}

if (top.myBrowser && top.myBrowser.needsXMLHttpRequestVariable) {var XMLHttpRequest;}